--                        -*- Mode: lua; lua-indent-level: 2; tab-width: 2 -*-
--[[
LibFilter_ClosetGnome

Author: Daphne Pfister <tiiroib02@sneakemail.com>

Thanks to Rabbit for writing ClosetGnome

License

Copyright (c) 2008 Daphne Pfister

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including
without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to
the following conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
]]

local MAJOR, MINOR = "LibFilter-item-ClosetGnome", tonumber(("$Revision: 29 $"):match("(%d+)"))
local filter = LibStub:NewLibrary(MAJOR, MINOR)
if not filter then return end

-- Avoid Global Lookups
local _G          = getfenv(0)
local ipairs      = _G.ipairs
local next        = _G.next
local pairs       = _G.pairs
local select      = _G.select
local string      = _G.string
local strjoin     = _G.strjoin
local table       = _G.table
local type        = _G.type
local unpack      = _G.unpack
local GetItemInfo = _G.GetItemInfo

local strmatch    = string.match
local tinsert     = table.insert
local tsort       = table.sort

-- Some icons
local ICON_EMPTY_CRATE = "Interface\\Icons\\INV_Crate_07"
local ICON_CLOSETGNOME = "Interface\\Icons\\INV_Chest_Cloth_17"
local ICON_QUESTION    = "Interface\\Icons\\INV_Misc_QuestionMark"
local ICON_WEAPONS     = "Interface\\Icons\\Achievement_Arena_5v5_3"

-- No need for a full blown locale library here
local L
do
	local LL = {
		enUS = {
			["ClosetGnome Set:%s"] = true,
			["Unknown"] = true,
			["ClosetGnome Sets"] = true,
			["Set Name"] = true,
			["Any"] = true,
			["None"] = true,
			["ClosetGnome: |cFFFF0000no sets defined|r"] = true,
			["ClosetGnome: |cFFFF0000not loaded|r"] = true,
			["ClosetGnome"] = true,
			["Filter by ClosetGnome set"] = true,
			["Fuzzy match"] = true,
			["Match by the item number, ignoring enchantments, variations, armor kits, etc."] = true,
		},
		koKR = {
			["ClosetGnome Set:%s"] = "ClosetGnome 세트명:%s",
			["Unknown"] = "가격미확인", -- check
			["ClosetGnome Sets"] = "ClosetGnome 세트",
			["Set Name"] = "세트명",
			["Any"] = "모두",
			["None"] = "없음", -- check
			--["ClosetGnome: |cFFFF0000no sets defined|r"] = true, -- fix
			--["ClosetGnome: |cFFFF0000not loaded|r"] = true, -- fix
			--["ClosetGnome"] = true,
			["Filter by ClosetGnome set"] = "ClosetGnome 세트에 따라 필터링",
			["Fuzzy match"] = "미세분화 분류",
			["Match by the item number, ignoring enchantments, variations, armor kits, etc."] = "마법부여나 키트 효과등을 무시하고 아이템 ID에 따라 분류합니다",
		},
		-- Thanks to swix
		roRU = {
			["ClosetGnome Set:%s"] = "Набор ClosetGnome:%s",
			["Unknown"] = "Неизвестно",
			["ClosetGnome Sets"] = "Наборы ClosetGnome",
			["Any"] = "Любой",
			["None"] = "Ни один",
			--["ClosetGnome: |cFFFF0000no sets defined|r"] = true, -- fix
			--["ClosetGnome: |cFFFF0000not loaded|r"] = true, -- fix
			["ClosetGnome"] = "ClosetGnome",
			["Filter by ClosetGnome set"] = "Фильтр по наборам ClosetGnome",
			["Fuzzy match"] = "Нечёткое соответствие",
			["Match by the item number, ignoring enchantments, variations, armor kits, etc."] = "Соответствие по номеру вещи, игнорирование зачарований, вариаций, коплектов брони и т.п.",
		},
	}

	local gGameLocale = GetLocale()
	local enL = LL["enUS"]
	L = LL[gGameLocale] or LL["enUS"]

	for vKey,vValue in pairs(enL) do
		if not L[vKey] then L[vKey] = vValue end
	end
	for vKey,vValue in pairs(L) do
		if vValue == true then L[vKey] = vKey end
	end
end

filter.author = "Nayala"
filter.credits = "ClosetGnome was written by Rabbit"
filter.description = L["Filter by ClosetGnome set"]
filter.filtertype = "item"
filter.key = "ClosetGnome"
filter.name = L["ClosetGnome"]
filter.icon = ICON_CLOSETGNOME

filter.callbacks = filter.callbacks or LibStub("CallbackHandler-1.0"):New(filter)
filter.events = filter.events or {}
filter.myFrame = filter.myFrame or CreateFrame("Frame","LibFilter_ClosetGnome_Frame")
local callbacks = filter.callbacks
local events = filter.events
local myFrame = filter.myFrame

local gItemsTable = nil
local gSetsTable = nil
local gSetIcons = nil

-- Prototypes
local Print -- (msg)
local LinkToIdAndItemNumber -- (pLink) -- returns id and fuzzy
local GetPlainItemNumber -- (pId)
local CacheSetItem -- (pSet, pItem, pTable1, pTable2, ...)
local CacheSets -- ()
local UpgradeSettings -- (pSettings)
local ExpandSettings -- (pSettings)
local NormalizeSettings -- (pSettings)
local Update
local HookClosetGnomeFunc -- (pFunc,pTarget)
local HookClosetGnome
local UpdateNextFrame
local ToggleSettingsSet -- (pSettings, pSet)
local ToggleSettingsFuzzy --- (pSettings)

function Print(msg)
	local info = ChatTypeInfo["SYSTEM"]
	DEFAULT_CHAT_FRAME:AddMessage(msg, info.r, info.g, info.b, info.id)
end

function LinkToIdAndItemNumber(pLink)
	if type(pLink) == "string" then
		local vFirst,vFuzzy,vLast = strmatch(pLink,"item:((%-?%d+):%-?%d+:%-?%d+:%-?%d+:%-?%d+:%-?%d+:%-?%d+:)(%-?%d+)")
		if vFirst and vLast then
			vLast = vLast % 65536
			return vFirst..vLast,vFuzzy
		end
	end
	return nil,nil
end

function GetPlainItemNumber(pId)
	if type(pId) == "string" then
		return strmatch(pId,"(%-?%d+)")
	else
		return pId
	end
end

function CacheSetItem(pSet, pItem, ...)
	if pItem then
		local vItemNumber = GetPlainItemNumber(pItem)

		for i=1,select("#",...) do
			local vCurrentTable = select(i,...)

			if vCurrentTable then
				if not vCurrentTable[pItem] then vCurrentTable[pItem] = {} end
				vCurrentTable[pItem][pSet] = true

				if vItemNumber ~= pItem then
					if not vCurrentTable[vItemNumber] then vCurrentTable[vItemNumber] = {} end
					vCurrentTable[vItemNumber][pSet] = true
				end
			end
		end
	end
end

local function ItemSlotValue(pLevel, pRarity)
	pLevel = 0 or pLevel
	pRarity = 0 or pRarity
  if pRarity == 0 then
		return (pLevel - 8) / 4 -- Poor
	elseif pRarity == 1 then
		return (pLevel - 8) / 3 -- Common
	elseif pRarity == 2 then
		return (pLevel - 8) / 2 -- Uncommon
	elseif pRarity == 3 then
		return (pLevel - 0.75) / 1.8 -- Rare
	elseif pRarity == 4 then
		return (pLevel - 26) / 1.2 -- Epic
	elseif pRarity > 4 then
		return pLevel -- Legendary or Artifact
	end
end

function CacheSets()
	local vSets = ClosetGnome.db.char.set

	gItemsTable = {}
	gSetsTable = {}
	gSetIcons = {}
	for vSetName, vSet in pairs(vSets) do
		--Print("Updating set: "..vSetName)
		gSetsTable[vSetName] = true
		local vBestIcon = nil
		local vBestValue = nil
		local vBestRarity = nil
		for i=0,19 do
			local vItem = vSet[i]
			if vItem then
				local vName,vLink,vRarity,vLevel,vMinLevel,vType,vSubType,vSCount,vEquipLoc,vIcon = GetItemInfo(vItem)
				if vIcon then
					local vValue = ItemSlotValue(vLevel,vRarity)
					if not vBestIcon or vValue > vBestValue then
						vBestIcon = vIcon
						vBestValue = vValue
					end
				end
				CacheSetItem(vSetName,LinkToIdAndItemNumber(vItem),gItemsTable)
			end
		end
		gSetIcons[vSetName] = vBestIcon or ICON_QUESTION
	end
end

function UpgradeSettings(pSettings)
	local vSets = pSettings.sets
	if not vSets then
		local vSet = pSettings.set
		if not vSet or vSet == "Any" then
			vSets = { Any = true }
		elseif vSet == "None" then
			vSets = { }
		else
			vSets = { [vSet] = true }
		end
		pSettings.sets = vSets
	end
	return vSets
end

function ExpandSettings(pSettings)
	local vSets = pSettings.sets or UpgradeSettings(pSettings)

	if vSets.Any and gSetsTable then
		for vSet in pairs(gSetsTable) do
			vSets[vSet] = true
		end
	end
	return vSets
end

function NormalizeSettings(pSettings)
	local vSets = pSettings.sets or UpgradeSettings(pSettings)

	local vNextSet = next(vSets)

	-- No sets selected
	if not vNextSet then
		pSettings.set = "None"
		return vSets
	end

	-- Just one set selected
	if not next(vSets,vNextSet) then
		pSettings.set = vNextSet
		return vSets
	end

	-- Okay check if all sets are selected
	if gSetsTable then
		vSets.Any = true
		for vSet in pairs(gSetsTable) do
			if not vSets[vSet] then vSets.Any = nil end
		end
	end

	if vSets.Any then
		vSets = { Any = true }
		pSettings.sets = vSets
		pSettings.set = "Any"
		return vSets
	end

	pSettings.set = "????"
	return vSets
end

myFrame:SetScript("OnEvent",function (self, event, ...) events[event](...) end)

function Update()
	callbacks:Fire("FilterUpdate")
end

function filter:CacheSetsAndUpdate()
	CacheSets()
	-- Should really only fire cache if there were changes
	callbacks:Fire("FilterUpdate")
	callbacks:Fire("PresetUpdate")
end

filter.hooked = filter.hooked or {}
local gHooked = filter.hooked

function HookClosetGnomeFunc(pFunc,pTarget)
	if not ClosetGnome then return end
	if not ClosetGnome.RegisterCallback then return end
	if gHooked[pFunc] ~= pTarget then
		ClosetGnome.RegisterCallback(filter,pFunc,pTarget)
		gHooked[pFunc] = pTarget
	end
end

local gIsHooked = nil
local gNeedCache = nil

function HookClosetGnome()
	if not ClosetGnome then return end
	if not ClosetGnome.db then return end
	if not ClosetGnome.db.char then return end

	if gIsHooked then return end
	gIsHooked = true

	HookClosetGnomeFunc("AddSet", "CacheSetsAndUpdate")
	HookClosetGnomeFunc("UpdateSet", "CacheSetsAndUpdate")
	HookClosetGnomeFunc("DeleteSet", "CacheSetsAndUpdate")
	gNeedCache = true
	UpdateNextFrame()
	myFrame:UnregisterEvent("ADDON_LOADED")
end

function events.UPDATE()
	myFrame:SetScript("OnUpdate",nil)
	if gNeedCache then
		filter:CacheSetsAndUpdate()
		gNeedCache = nil
	end
	HookClosetGnome()
end

function UpdateNextFrame()
	myFrame:SetScript("OnUpdate",function (self, ...) events.UPDATE(...) end)
end

function events.ADDON_LOADED()
	UpdateNextFrame()
end

function events.PLAYER_LOGIN()
	UpdateNextFrame()
	myFrame:RegisterEvent("ADDON_LOADED")
end

if IsLoggedIn() then
	UpdateNextFrame()
else
	myFrame:RegisterEvent("PLAYER_LOGIN")
end

local function NextPreset(pUnused, pPreset)
	if pPreset == "" then
		pPreset = nil
	elseif gSetsTable and not pPreset then
		return "", ""
	end
	if gSetsTable then pPreset = next(gSetsTable,pPreset) end
	return pPreset,pPreset and ""
end

function filter:IteratePresets(pSettings)
  return NextPreset, nil, nil
end

function filter:NewSettings(pSettings)
	local vRes
	if pSettings == "" then
		vRes = {
			set = "Any",
			fuzzy = false,
		}
	elseif type(pSettings) == "string" then
		vRes = {
			set = pSettings,
			fuzzy = false,
		}
	elseif type(pSettings) == "table" then
		vRes = {
			set = pSettings and pSettings.set,
			fuzzy = pSettings and pSettings.fuzzy or false,
		}
	else
		vRes = {
			set = "Any",
			fuzzy = false,
		}
		if pSettings and pSettings.sets then
			local vSets = {}
			for vKey,vValue in pairs(pSettings.sets) do
				vSets[vKey] = vValue
			end
			vRes.sets = vSets
		end
	end
	if not vRes.sets then
		UpgradeSettings(vRes)
	end
	return vRes
end

function filter:FreeSettings(pSettings)
end

function filter:DescribeSettings(pSettings)
	local vSetList
	local vIcon

	if pSettings == "" then
		vSetList = L["Any"]
		vIcon = ICON_CLOSETGNOME
	elseif type(pSettings) == "string" then
		vSetList = pSettings
		vIcon = gSetIcons and gSetIcons[pSettings] or ICON_QUESTION
	else
		local vSets = pSettings.sets or UpgradeSettings(pSettings)
		if not next(vSets) then
			vSetList = L["None"]
			vIcon = ICON_EMPTY_CRATE
		elseif vSets.Any then
			vSetList = L["Any"]
			vIcon = ICON_CLOSETGNOME
		else
			local vSetNames = {}
			for vSet in pairs(vSets) do
				tinsert(vSetNames,vSet)
				local vSetIcon = gSetIcons and gSetIcons[vSet] or ICON_QUESTION
				if not vIcon then
					vIcon = vSetIcon
				elseif vIcon ~= vSetIcon then
					vIcon = ICON_WEAPONS
				end
			end
			if not vIcon then
				vIcon = ICON_QUESTION
			end
			tsort(vSetNames)
			vSetList = strjoin(", ",unpack(vSetNames))
		end
	end

	local vDescription = L["ClosetGnome Set:%s"]:format(vSetList)
	if pSettings.fuzzy then
		vDescription = vDescription.." "..L["Fuzzy match"]
	end
	return vDescription,vIcon,vSetList
end

function ToggleSettingsSet(pSettings, pSet)
	if pSet == "Any" then
		pSettings.set = "Any"
		pSettings.sets = { Any = true }
	elseif pSet == "None" then
		pSettings.set = "None"
		pSettings.sets = { }
	else
		local vSets = ExpandSettings(pSettings)

		-- Okay update the given set
		vSets[pSet] = not vSets[pSet] and true or nil

		-- Okay normalize
		vSets.Any = nil -- Since we are expanded, can clear Any
		NormalizeSettings(pSettings)
	end
end

function toggleSettingsFuzzy(pSettings)
	pSettings.fuzzy = not pSettings.fuzzy
end

function filter:SettingsOptions(pSettings,pTable,pMethod)
	local vCallback = Update

	if type(pSettings) ~= "table" then return end

	if pTable then
		if type(pTable) == "string" then
			vCallback = function() pTable[pMethod](pTable,pSettings) end
		else
			vCallback = function() pTable(pSettings) end
		end
	end

	local vOptions = {}
	local vMain = {
		type = "group",
		desc = "LibFilter_ClosetGnome options",
		args = vOptions
	}

	local vSets = {}
	if gSetsTable then
		if next(gSetsTable) then
			-- Would rather have used a multiselect here -- but not everything supports it
			for vSet in pairs(gSetsTable) do
				vSets[vSet] = {
					type = "toggle",
					name = vSet,
					desc = L["ClosetGnome Set:%s"]:format(vSet),
					icon = gSetIcons[vSet] or nil,
					get = function(pInfo) return pSettings.sets.Any or pSettings.sets[vSet] end,
					set = function(pInfo, pValue)
									ToggleSettingsSet(pSettings,vSet)
									vCallback()
								end,
				}
			end
		else
			vOptions[1] = {
				order = 1,
				name = L["ClosetGnome: |cFFFF0000no sets defined|r"],
				desc = L["ClosetGnome: |cFFFF0000no sets defined|r"],
				type = "header"
			}
			return vMain
		end
	else
		vOptions[1] = {
			order = 1,
			name = L["ClosetGnome: |cFFFF0000not loaded|r"],
			desc = L["ClosetGnome: |cFFFF0000not loaded|r"],
			type = "header"
		}
		return vMain
	end

	vOptions[1] = {
		order = 1,
		type = "toggle",
		name = L["Any"],
		desc = L["Any"],
		get = function() return pSettings.sets.Any end,
		set = function(pInfo, pV)
						ToggleSettingsSet(pSettings,"Any")
						vCallback()
					end,
	}
	vOptions[2] = {
		order = 2,
		type = "toggle",
		name = L["None"],
		desc = L["None"],
		get = function() return not next(pSettings.sets) end,
		set = function(pInfo, pV)
						ToggleSettingsSet(pSettings,"None")
						vCallback()
					end,
	}
	vOptions[3] = {
		order = 3,
		name = L["ClosetGnome Sets"],
		desc = L["ClosetGnome Sets"],
		type = "group", -- Would prefer this being a multiselect
		args = vSets,
	}
	vOptions[4] = {
		order = 4,
		type = "toggle",
		name = L["Fuzzy match"],
		desc = L["Match by the item number, ignoring enchantments, variations, armor kits, etc."],
		get = function() return pSettings.fuzzy end,
		set = function(pInfo, pV)
						ToggleSettingsFuzzy(pSettings)
						vCallback()
					end,
	}
	return vMain
end

function filter:TestIfMatch(settings,link)
	local id,idfuzzy = LinkToIdAndItemNumber(link)
	if not id then return end
	local res = gItemsTable

	if type(settings) == "table" then
		if settings.fuzzy then
			id = idfuzzy
			if not id then return end
		end
	end

	if res then
		if id and res[id] then
			res = res[id]
		else
			res = nil
		end
	end

	if type(settings) == "string" then
		if settings ~= "" then
			res = res and res[settings]
		end
		return res
	else
		local sets = settings.sets or UpgradeSettings(settings)
		local any = sets.Any
		local none = not next(sets)

		if none then
			return not res
		elseif any or not res then
			return res
		else
			for set in pairs(sets) do
				if res[set] then return res end
			end
		end
	end
	return nil
end
