--                        -*- Mode: lua; lua-indent-level: 2; tab-width: 2 -*-
--[[
LibFilter_EquipSlot

Author: Daphne Pfister <tiiroib02@sneakemail.com>

License

Copyright (c) 2008 Daphne Pfister

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including
without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to
the following conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
]]

local MAJOR, MINOR = "LibFilter-item-EquipSlot", tonumber(("$Revision: 34 $"):match("(%d+)"))
local filter = LibStub:NewLibrary(MAJOR, MINOR)
if not filter then return end

-- Avoid Global Lookups
local _G               = getfenv(0)
local ipairs           = _G.ipairs
local next             = _G.next
local pairs            = _G.pairs
local select           = _G.select
local string           = _G.string
local strjoin          = _G.strjoin
local table            = _G.table
local type             = _G.type
local unpack           = _G.unpack
local GetItemInfo      = _G.GetItemInfo;

local strmatch         = string.match
local tinsert          = table.insert
local tsort            = table.sort

local INVTYPE_TO_SLOT = {
  INVTYPE_AMMO                  = "AMMOSLOT",
  INVTYPE_HEAD                  = "HEADSLOT",
  INVTYPE_NECK                  = "NECKSLOT",
  INVTYPE_SHOULDER              = "SHOULDERSLOT",
  INVTYPE_BODY                  = "CHESTSLOT",
  INVTYPE_CHEST                 = "CHESTSLOT",
  INVTYPE_ROBE                  = "CHESTSLOT",
  INVTYPE_WAIST                 = "WAISTSLOT",
  INVTYPE_LEGS                  = "LEGSSLOT",
  INVTYPE_FEET                  = "FEETSLOT",
  INVTYPE_WRIST                 = "WRISTSLOT",
  INVTYPE_HAND                  = "HANDSSLOT",
  INVTYPE_FINGER                = "FINGER0SLOT",
  INVTYPE_TRINKET               = "TRINKET0SLOT",
  INVTYPE_CLOAK                 = "BACKSLOT",
  INVTYPE_WEAPON                = "MAINHANDSLOT", -- and SECONDARYHANDSLOT
  INVTYPE_SHIELD                = "SECONDARYHANDSLOT",
  INVTYPE_2HWEAPON              = "MAINHANDSLOT",
  INVTYPE_WEAPONMAINHAND        = "MAINHANDSLOT",
  INVTYPE_WEAPONOFFHAND         = "SECONDARYHANDSLOT",
  INVTYPE_HOLDABLE              = "SECONDARYHANDSLOT",
  INVTYPE_RANGED                = "RANGEDSLOT",
  INVTYPE_THROWN                = "RANGEDSLOT",
  INVTYPE_RANGEDRIGHT           = "RANGEDSLOT",
  INVTYPE_RELIC                 = "RANGEDSLOT",
  INVTYPE_TABARD                = "TABARDSLOT",
  INVTYPE_BAG                   = "BAG0SLOT",
  INVTYPE_QUIVER                = "BAG0SLOT",
}

local PRESETS = {}
local ICONS = {}
local DESCRIPTIONS = {}

local ICON_EQUIPSLOT = "Interface\\PaperDoll\\UI-PaperDoll-Slot-Chest"

-- No need for a full blown locale library here
local L
do
	local LL = {
		enUS = {
			["EquipSlot"] = "EquipSlot", -- Filter name
			["Filter by equipment slot"] = true,
			["Slots: %s"] = true,
		},
	}

	local gameLocale = GetLocale()
	local enL = LL["enUS"]
	L = LL[gameLocale] or LL["enUS"]

	for k in pairs(enL) do
		if not LL[k] then LL[k] = k end
	end
	for k,v in pairs(L) do
		if v == true then L[k] = k end
	end
end

filter.author = "Nayala"
filter.credits = ""
filter.description = L["Filter by equipment slot"]
filter.filtertype = "item"
filter.filtertypes = filter.filtertypes or {}
filter.filtertypes["item"] = 1

filter.key = "EquipSlot"
filter.name = L["EquipSlot"]
filter.icon = ICON_EQUIPSLOT

filter.callbacks = filter.callbacks or LibStub("CallbackHandler-1.0"):New(filter)
local callbacks = filter.callbacks


for invtype,slot in pairs(INVTYPE_TO_SLOT) do
	PRESETS[slot] = ""
end

DESCRIPTIONS.BAG0SLOT = _G.BAGSLOT

function Print(msg)
	local info = ChatTypeInfo["SYSTEM"]
	DEFAULT_CHAT_FRAME:AddMessage(msg, info.r, info.g, info.b, info.id)
end

for slot in pairs(PRESETS) do
	Print(slot)
	ICONS[slot] = select(2, GetInventorySlotInfo(slot))
	DESCRIPTIONS[slot] = DESCRIPTIONS[slot] or _G[slot]
end

function filter:IteratePresets()
	return pairs(PRESETS)
end

function filter:NewSettings(pSettings)
	local vRes = {}
	if type(pSettings) == "string" then
		vRes[pSettings] = true
	elseif type(pSettings) == "table" then
		for state, value in pairs(pSettings) do
			vRes[state] = value
		end
	end
	return vRes
end

function filter:FreeSettings(pSettings)
end

function filter:DescribeSettings(pSettings)
	local vStateList
	local vIcon

	if pSettings == "" then return filter.name,ICON_EQUIPSLOT,filter.name end

	if type(pSettings) == "string" then
		vStateList = DESCRIPTIONS[pSettings]
		vIcon = ICONS[pSettings] or ICON_EQUIPSLOT
	else
		local vStateNames = {}
		vIcon = nil
		for vState, vValue in pairs(pSettings) do
			if vValue then
				tinsert(vStateNames,DESCRIPTIONS[vState])
				local icon = ICONS[vState]
				if not vIcon then vIcon = icon end
				if vIcon ~= icon then vIcon = ICON_EQUIPSLOT end
			end
		end
		tsort(vStateNames)
		vStateList = strjoin(", ",unpack(vStateNames))
		if not vIcon then vIcon = ICON_EQUIPSLOT end
	end

	return L["Slots: %s"]:format(vStateList),vIcon,vStateList
end

local function update()
	callbacks:Fire("FilterUpdate")
end

function filter:SettingsOptions(pSettings,pTable,pMethod)
	local vCallback = Update

	if type(pSettings) ~= "table" then return end

	if pTable then
		if type(pTable) == "string" then
			vCallback = function() pTable[pMethod](pTable,pSettings) end
		else
			vCallback = function() pTable(pSettings) end
		end
	end

	local vOptions = {}
	local vMain = {
		type = "group",
		desc = "LibFilter_EquipSlot options",
		args = vOptions
	}

	local vOptions = {}
	for vSlot,vIcon in pairs(ICONS) do
		vOptions[vSlot] = {
			type = "toggle",
			name = DESCRIPTIONS[vSlot],
			desc = DESCRIPTIONS[vSlot],
			icon = vIcon,
			get = function(pInfo) return pSettings[vSlot] end,
			set = function(pInfo, pValue)
							pSettings[vSlot] = not pSettings[vSlot],
							vCallback()
						end,
		}
	end
	return vMain
end

function filter:TestIfMatch(settings,link)
	if not link then return end

	local _,equicloc
	_,_,_,_,_,_,_,_,equiploc = GetItemInfo(link)

	local slot = INVTYPE_TO_SLOT[equiploc]
	local altslot = slot
	if equiploc == "INVTYPE_WEAPON" then altslot = "SECONDARYHANDSLOT" end

	if type(settings) == "string" then
		return settings == slot or settings == altslot
	else
		return settings[slot] or settings[altslot]
	end
end
